@@ -2,306 +2,413 @@ info = {
     "name": "uk",
     "date_order": "DMY",
     "january": [
-        "січень",
         "січ",
+        "січень",
         "січня"
     ],
     "february": [
-        "лютий",
         "лют",
+        "лютий",
         "лютого"
     ],
     "march": [
-        "березень",
         "бер",
+        "берез",
+        "березень",
         "березня"
     ],
     "april": [
-        "квітень",
         "кві",
-        "квітня",
-        "квіт"
+        "квіт",
+        "квітень",
+        "квітня"
     ],
     "may": [
-        "травень",
         "тра",
-        "травня",
-        "трав"
+        "трав",
+        "травень",
+        "травня"
     ],
     "june": [
-        "червень",
         "чер",
-        "червня",
-        "черв"
+        "черв",
+        "червень",
+        "червня"
     ],
     "july": [
-        "липень",
         "лип",
+        "липень",
         "липня"
     ],
     "august": [
-        "серпень",
         "сер",
-        "серпня",
-        "серп"
+        "серп",
+        "серпень",
+        "серпня"
     ],
     "september": [
-        "вересень",
         "вер",
+        "верес",
+        "вересень",
         "вересня"
     ],
     "october": [
-        "жовтень",
         "жов",
-        "жовтня",
-        "жовт"
+        "жовт",
+        "жовтень",
+        "жовтня"
     ],
     "november": [
-        "листопад",
         "лис",
-        "листопада",
-        "лист"
+        "лист",
+        "листоп",
+        "листопад",
+        "листопада"
     ],
     "december": [
-        "грудень",
         "гру",
-        "грудня",
-        "груд"
+        "груд",
+        "грудень",
+        "грудня"
     ],
     "monday": [
+        "пн",
+        "пон",
+        "понед",
         "понеділок",
         "пн",
-        "пон"
+        "пон",
+        "понед",
+        "понеділок"
     ],
     "tuesday": [
+        "вт",
+        "вів",
         "вівторок",
         "вт",
-        "вів"
+        "вів",
+        "вівтор",
+        "вівторок"
     ],
     "wednesday": [
+        "ср",
+        "сер",
         "середа",
-        "ср"
+        "ср",
+        "сер",
+        "серед",
+        "середа",
+        "середу"
     ],
     "thursday": [
+        "чт",
+        "чет",
         "четвер",
         "чт",
-        "чет"
+        "чет",
+        "четвер"
     ],
     "friday": [
-        "п'ятниця",
-        "пт"
+        "пт",
+        "пят",
+        "п'ятниця"
     ],
     "saturday": [
+        "сб",
+        "суб",
         "субота",
         "сб",
-        "суб"
+        "суб",
+        "субот",
+        "субота",
+        "суботу"
     ],
     "sunday": [
+        "нд",
+        "нед",
         "неділя",
         "нд",
-        "нед"
+        "нед",
+        "неділ",
+        "неділя",
+        "неділю"
     ],
     "am": [
-        "дп"
+        "дп",
+        "до полудня"
     ],
     "pm": [
-        "пп"
+        "пп",
+        "після полудня",
+        "пополудні",
+        "пополудні"
     ],
     "year": [
-        "рік",
         "р",
+        "рік",
+        "року",
+        "років",
+        "роками",
+        "роках",
         "роки",
-        "років"
+        "рокові",
+        "роком",
+        "році",
+        "р",
+        "рік",
+        "роки",
+        "років",
+        "року",
+        "роком",
+        "році"
     ],
     "month": [
-        "місяць",
         "міс",
+        "місяць",
+        "місяця",
+        "місяців",
+        "місяцями",
+        "місяцях",
         "місяці",
-        "місяців"
+        "місяцеві",
+        "місяцю",
+        "місяцем",
+        "м",
+        "міс",
+        "місяць",
+        "місяці",
+        "місяців",
+        "місяця",
+        "місяцю",
+        "місяцям",
+        "місяцем"
     ],
     "week": [
-        "тиждень",
         "тиж",
+        "тижня",
+        "тижневі",
+        "тижню",
+        "тижнем",
+        "тижню",
+        "тиждень",
+        "т",
+        "тиждень",
+        "тижня",
         "тижні",
-        "тижнів"
+        "тижнів",
+        "тижнем"
     ],
     "day": [
+        "д",
+        "дня",
+        "дневі",
+        "дню",
+        "днем",
         "день",
         "д",
+        "дн",
+        "день",
+        "дня",
         "дні",
-        "днів"
+        "днів",
+        "доба",
+        "доби",
+        "діб"
     ],
     "hour": [
-        "година",
         "год",
+        "година",
+        "годині",
+        "годиною",
+        "годино",
+        "г",
+        "год",
+        "годин",
+        "година",
         "години",
         "годин",
         "годину",
         "г"
     ],
     "minute": [
+        "хв",
         "хвилина",
+        "хвилині",
+        "хвилиною",
+        "хвилино",
         "хв",
         "хвилини",
         "хвилин",
         "хвилину"
     ],
     "second": [
-        "секунда",
         "с",
+        "секунда",
+        "секунді",
+        "секундою",
+        "секундо",
+        "с",
+        "секунда",
         "секунди",
         "секунд",
         "секунду",
         "сек"
     ],
     "relative-type": {
-        "1 year ago": [
-            "торік"
-        ],
-        "0 year ago": [
-            "цього року"
-        ],
-        "in 1 year": [
-            "наступного року"
-        ],
-        "1 month ago": [
-            "минулого місяця"
-        ],
-        "0 month ago": [
-            "цього місяця"
-        ],
-        "in 1 month": [
-            "наступного місяця"
-        ],
-        "1 week ago": [
-            "минулого тижня"
-        ],
-        "0 week ago": [
-            "цього тижня"
-        ],
-        "in 1 week": [
-            "наступного тижня"
-        ],
-        "1 day ago": [
-            "учора",
-            "вчора"
-        ],
         "0 day ago": [
             "сьогодні"
         ],
-        "in 1 day": [
-            "завтра"
-        ],
         "0 hour ago": [
             "цієї години"
         ],
         "0 minute ago": [
             "цієї хвилини"
         ],
+        "0 month ago": [
+            "цього місяця"
+        ],
         "0 second ago": [
             "зараз"
         ],
+        "0 week ago": [
+            "цього тижня"
+        ],
+        "0 year ago": [
+            "цього року"
+        ],
+        "1 day ago": [
+            "учора",
+            "вчора"
+        ],
+        "1 month ago": [
+            "минулого місяця"
+        ],
+        "1 week ago": [
+            "минулого тижня"
+        ],
+        "1 year ago": [
+            "торік"
+        ],
+        "in 1 day": [
+            "завтра"
+        ],
+        "in 1 month": [
+            "наступного місяця"
+        ],
+        "in 1 week": [
+            "наступного тижня"
+        ],
+        "in 1 year": [
+            "наступного року"
+        ],
         "2 day ago": [
             "позавчора"
+        ],
+        "in 2 day": [
+            "післязавтра"
         ]
     },
     "relative-type-regex": {
-        "in \\1 year": [
-            "через (\\d+) рік",
-            "через (\\d+) року",
-            "через (\\d+) р",
-            "за (\\d+) р"
-        ],
-        "\\1 year ago": [
-            "(\\d+) рік тому",
-            "(\\d+) року тому",
-            "(\\d+) р тому"
-        ],
-        "in \\1 month": [
-            "через (\\d+) місяць",
-            "через (\\d+) місяця",
-            "через (\\d+) міс",
-            "за (\\d+) міс"
-        ],
-        "\\1 month ago": [
-            "(\\d+) місяць тому",
-            "(\\d+) місяця тому",
-            "(\\d+) міс тому"
-        ],
-        "in \\1 week": [
-            "через (\\d+) тиждень",
-            "через (\\d+) тижня",
-            "через (\\d+) тиж",
-            "за (\\d+) тиж"
-        ],
-        "\\1 week ago": [
-            "(\\d+) тиждень тому",
-            "(\\d+) тижня тому",
-            "(\\d+) тиж тому"
-        ],
-        "in \\1 day": [
-            "через (\\d+) день",
-            "через (\\d+) дня",
-            "через (\\d+) дн",
-            "за (\\d+) д"
-        ],
         "\\1 day ago": [
+            "(\\d+) д тому",
             "(\\d+) день тому",
-            "(\\d+) дня тому",
             "(\\d+) дн тому",
-            "(\\d+) д тому"
-        ],
-        "in \\1 hour": [
-            "через (\\d+) годину",
-            "через (\\d+) години",
-            "через (\\d+) год",
-            "за (\\d+) год"
+            "(\\d+) дня тому"
         ],
         "\\1 hour ago": [
-            "(\\d+) годину тому",
+            "(\\d+) год тому",
             "(\\d+) години тому",
-            "(\\d+) год тому"
-        ],
-        "in \\1 minute": [
-            "через (\\d+) хвилину",
-            "через (\\d+) хвилини",
-            "через (\\d+) хв",
-            "за (\\d+) хв"
+            "(\\d+) година тому",
+            "(\\d+) годину тому"
         ],
         "\\1 minute ago": [
-            "(\\d+) хвилину тому",
+            "(\\d+) хв тому",
+            "(\\d+) хвилина тому",
             "(\\d+) хвилини тому",
-            "(\\d+) хв тому"
+            "(\\d+) хвилину тому"
         ],
-        "in \\1 second": [
-            "через (\\d+) секунду",
-            "через (\\d+) секунди",
-            "через (\\d+) с",
-            "за (\\d+) с"
+        "\\1 month ago": [
+            "(\\d+) міс тому",
+            "(\\d+) місяць тому",
+            "(\\d+) місяця тому"
         ],
         "\\1 second ago": [
-            "(\\d+) секунду тому",
+            "(\\d+) с тому",
+            "(\\d+) секунда тому",
             "(\\d+) секунди тому",
-            "(\\d+) с тому"
+            "(\\d+) секунду тому"
+        ],
+        "\\1 week ago": [
+            "(\\d+) тиж тому",
+            "(\\d+) тиждень тому",
+            "(\\d+) тижня тому"
+        ],
+        "\\1 year ago": [
+            "(\\d+) р тому",
+            "(\\d+) року тому",
+            "(\\d+) рік тому"
+        ],
+        "in \\1 day": [
+            "за (\\d+) д",
+            "через (\\d+) день",
+            "через (\\d+) дн",
+            "через (\\d+) дня"
+        ],
+        "in \\1 hour": [
+            "за (\\d+) год",
+            "через (\\d+) год",
+            "через (\\d+) години",
+            "через (\\d+) годину"
+        ],
+        "in \\1 minute": [
+            "за (\\d+) хв",
+            "через (\\d+) хв",
+            "через (\\d+) хвилини",
+            "через (\\d+) хвилину"
+        ],
+        "in \\1 month": [
+            "за (\\d+) міс",
+            "через (\\d+) міс",
+            "через (\\d+) місяць",
+            "через (\\d+) місяця"
+        ],
+        "in \\1 second": [
+            "за (\\d+) с",
+            "через (\\d+) с",
+            "через (\\d+) секунди",
+            "через (\\d+) секунду"
+        ],
+        "in \\1 week": [
+            "за (\\d+) тиж",
+            "через (\\d+) тиж",
+            "через (\\d+) тиждень",
+            "через (\\d+) тижня"
+        ],
+        "in \\1 year": [
+            "за (\\d+) р",
+            "через (\\d+) р",
+            "через (\\d+) року",
+            "через (\\d+) рік"
         ]
     },
     "locale_specific": {},
     "skip": [
+        "приблизно",
         "близько",
         "о",
         "об",
         "і",
         " ",
-        ".",
-        ",",
-        ";",
-        "-",
-        "/",
         "'",
-        "|",
+        ",",
+        "-",
+        ".",
+        "/",
+        ";",
         "@",
         "[",
         "]",
+        "|",
         "，"
     ],
     "sentence_splitter_group": 1,
@@ -310,9 +417,83 @@ info = {
         "назад"
     ],
     "in": [
-        "протягом"
+        "протягом",
+        "через",
+        "за"
     ],
     "simplifications": [
+        {
+            "од(на|ну|ні|ної|нієї|ин|ній)": "1"
+        },
+        {
+            "дв(а|і|оє|ох)": "2"
+        },
+        {
+            "пар[ауі]": "2"
+        },
+        {
+            "три": "3"
+        },
+        {
+            "чотири": "4"
+        },
+        {
+            "п'ять": "5"
+        },
+        {
+            "шість": "6"
+        },
+        {
+            "сім": "7"
+        },
+        {
+            "вісім": "8"
+        },
+        {
+            "дев'ять": "9"
+        },
+        {
+            "десять": "10"
+        },
+        {
+            "одинадцять": "11"
+        },
+        {
+            "дванадцять": "12"
+        },
+        {
+            "п'ятнадцять": "15"
+        },
+        {
+            "двадцять": "20"
+        },
+        {
+            "тридцять": "30"
+        },
+        {
+            "сорок": "40"
+        },
+        {
+            "п'ятдесят": "50"
+        },
+        {
+            "півгодини": "30 хвилин"
+        },
+        {
+            "пів години": "30 хвилин"
+        },
+        {
+            "півроку": "6 місяців"
+        },
+        {
+            "пів року": "6 місяців"
+        },
+        {
+            "півтори години": "90 хвилин"
+        },
+        {
+            "півтора року": "18 місяців"
+        },
         {
             "^година": "1 година"
         },
