@@ -4,6 +4,7 @@ from collections.abc import Set
 from datetime import datetime, timedelta
 
 import regex as re
+from tzlocal import get_localzone
 from dateutil.relativedelta import relativedelta
 
 from dateparser.date_parser import date_parser
@@ -13,7 +14,8 @@ from dateparser.conf import apply_settings, check_settings
 from dateparser.parser import _parse_absolute, _parse_nospaces
 from dateparser.timezone_parser import pop_tz_offset_from_string
 from dateparser.utils import apply_timezone_from_settings, \
-    set_correct_day_from_settings
+    set_correct_day_from_settings, \
+    get_timezone_from_tz_string
 from dateparser.custom_language_detection.language_mapping import map_languages
 
 APOSTROPHE_LOOK_ALIKE_CHARS = [
@@ -33,13 +35,14 @@ RE_SPACES = re.compile(r'\s+')
 RE_TRIM_SPACES = re.compile(r'^\s+(\S.*?)\s+$')
 RE_TRIM_COLONS = re.compile(r'(\S.*?):*$')
 
-RE_SANITIZE_SKIP = re.compile(r'\t|\n|\r|\u00bb|,\s\u0432|\u200e|\xb7|\u200f|\u064e|\u064f', flags=re.M)
+RE_SANITIZE_SKIP = re.compile(r'\t|\n|\r|\u00bb|,\s\u0432\b|\u200e|\xb7|\u200f|\u064e|\u064f', flags=re.M)
 RE_SANITIZE_RUSSIAN = re.compile(r'([\W\d])\u0433\.', flags=re.I | re.U)
-RE_SANITIZE_PERIOD = re.compile(r'(?<=\D+)\.', flags=re.U)
+RE_SANITIZE_PERIOD = re.compile(r'(?<=[^0-9\s])\.', flags=re.U)
 RE_SANITIZE_ON = re.compile(r'^.*?on:\s+(.*)')
 RE_SANITIZE_APOSTROPHE = re.compile('|'.join(APOSTROPHE_LOOK_ALIKE_CHARS))
 
 RE_SEARCH_TIMESTAMP = re.compile(r'^(\d{10})(\d{3})?(\d{3})?(?![^.])')
+RE_SEARCH_NEGATIVE_TIMESTAMP = re.compile(r'^([-]\d{10})(\d{3})?(\d{3})?(?![^.])')
 
 
 def sanitize_spaces(date_string):
@@ -112,14 +115,30 @@ def sanitize_date(date_string):
     return date_string
 
 
-def get_date_from_timestamp(date_string, settings):
-    match = RE_SEARCH_TIMESTAMP.search(date_string)
+def get_date_from_timestamp(date_string, settings, negative=False):
+    if negative:
+        match = RE_SEARCH_NEGATIVE_TIMESTAMP.search(date_string)
+    else:
+        match = RE_SEARCH_TIMESTAMP.search(date_string)
+
     if match:
+        if (settings is None or
+            settings.TIMEZONE is None or
+            'local' in settings.TIMEZONE.lower()):
+            # If the timezone in settings is unset, or it's 'local', use the
+            # local timezone
+            timezone = get_localzone()
+        else:
+            # Otherwise, use the timezone given in settings
+            timezone = get_timezone_from_tz_string(settings.TIMEZONE)
+
         seconds = int(match.group(1))
         millis = int(match.group(2) or 0)
         micros = int(match.group(3) or 0)
-        date_obj = datetime.fromtimestamp(seconds)
-        date_obj = date_obj.replace(microsecond=millis * 1000 + micros)
+        date_obj = (datetime
+                    .fromtimestamp(seconds, timezone)
+                    .replace(microsecond=millis * 1000 + micros, tzinfo=None)
+                    )
         date_obj = apply_timezone_from_settings(date_obj, settings)
         return date_obj
 
@@ -166,6 +185,7 @@ class _DateLocaleParser:
         self._translated_date_with_formatting = None
         self._parsers = {
             'timestamp': self._try_timestamp,
+            'negative-timestamp': self._try_negative_timestamp,
             'relative-time': self._try_freshness_parser,
             'custom-formats': self._try_given_formats,
             'absolute-time': self._try_absolute_parser,
@@ -185,12 +205,18 @@ class _DateLocaleParser:
         else:
             return None
 
-    def _try_timestamp(self):
+    def _try_timestamp_parser(self, negative=False):
         return DateData(
-            date_obj=get_date_from_timestamp(self.date_string, self._settings),
+            date_obj=get_date_from_timestamp(self.date_string, self._settings, negative=negative),
             period='time' if self._settings.RETURN_TIME_AS_PERIOD else 'day',
         )
 
+    def _try_timestamp(self):
+        return self._try_timestamp_parser()
+
+    def _try_negative_timestamp(self):
+        return self._try_timestamp_parser(negative=True)
+
     def _try_freshness_parser(self):
         try:
             return freshness_date_parser.get_date_data(self._get_translated_date(), self._settings)
@@ -358,8 +384,8 @@ class DateDataParser:
             raise TypeError("use_given_order argument must be a boolean (%r given)"
                             % type(use_given_order))
 
-        if not locales and use_given_order:
-            raise ValueError("locales must be given if use_given_order is True")
+        if not locales and not languages and use_given_order:
+            raise ValueError("locales or languages must be given if use_given_order is True")
 
         check_settings(settings)
 
