@@ -1,4 +1,3 @@
-# coding: utf-8
 import calendar
 import regex as re
 
@@ -6,20 +5,25 @@ from io import StringIO
 from collections import OrderedDict
 from datetime import datetime
 from datetime import timedelta
+from pytz import utc
 
+from dateparser.utils import set_correct_day_from_settings, \
+    get_last_day_of_month, get_previous_leap_year, get_next_leap_year, \
+    _get_missing_parts
 from dateparser.utils.strptime import strptime
 
 
 NSP_COMPATIBLE = re.compile(r'\D+')
 MERIDIAN = re.compile(r'am|pm')
 MICROSECOND = re.compile(r'\d{1,6}')
+EIGHT_DIGIT = re.compile(r'^\d{8}$')
+HOUR_MINUTE_REGEX = re.compile(r'^([0-9]|0[0-9]|1[0-9]|2[0-3]):[0-5][0-9]$')
 
 
 def no_space_parser_eligibile(datestring):
     src = NSP_COMPATIBLE.search(datestring)
     if not src or ':' == src.group():
         return True
-
     return False
 
 
@@ -35,42 +39,39 @@ def get_unresolved_attrs(parser_object):
     return seen, unseen
 
 
+date_order_chart = {
+    'DMY': '%d%m%y',
+    'DYM': '%d%y%m',
+    'MDY': '%m%d%y',
+    'MYD': '%m%y%d',
+    'YDM': '%y%d%m',
+    'YMD': '%y%m%d',
+}
+
+
 def resolve_date_order(order, lst=None):
-    chart = {
-        'MDY': '%m%d%y',
-        'MYD': '%m%y%d',
-        'YMD': '%y%m%d',
-        'YDM': '%y%d%m',
-        'DMY': '%d%m%y',
-        'DYM': '%d%y%m',
-    }
 
     chart_list = {
-        'MDY': ['month', 'day', 'year'],
-        'MYD': ['month', 'year', 'day'],
-        'YMD': ['year', 'month', 'day'],
-        'YDM': ['year', 'day', 'month'],
         'DMY': ['day', 'month', 'year'],
         'DYM': ['day', 'year', 'month'],
+        'MDY': ['month', 'day', 'year'],
+        'MYD': ['month', 'year', 'day'],
+        'YDM': ['year', 'day', 'month'],
+        'YMD': ['year', 'month', 'day'],
     }
 
-    return chart_list[order] if lst else chart[order]
+    return chart_list[order] if lst else date_order_chart[order]
 
 
-def parse(datestring, settings):
-    exceptions = []
-    for parser in [_parser.parse, _no_spaces_parser.parse]:
-        try:
-            res = parser(datestring, settings)
-            if res:
-                return res
-        except Exception as e:
-            exceptions.append(e)
-    else:
-        raise exceptions.pop(-1)
+def _parse_absolute(datestring, settings):
+    return _parser.parse(datestring, settings)
 
 
-class _time_parser(object):
+def _parse_nospaces(datestring, settings):
+    return _no_spaces_parser.parse(datestring, settings)
+
+
+class _time_parser:
     time_directives = [
         '%H:%M:%S',
         '%I:%M:%S %p',
@@ -79,6 +80,7 @@ class _time_parser(object):
         '%I %p',
         '%H:%M:%S.%f',
         '%I:%M:%S.%f %p',
+        '%H:%M %p'
     ]
 
     def __call__(self, timestring):
@@ -91,10 +93,11 @@ class _time_parser(object):
         else:
             raise ValueError('%s does not seem to be a valid time string' % _timestring)
 
+
 time_parser = _time_parser()
 
 
-class _no_spaces_parser(object):
+class _no_spaces_parser:
     _dateformats = [
         '%Y%m%d', '%Y%d%m', '%m%Y%d',
         '%m%d%Y', '%d%Y%m', '%d%m%Y',
@@ -104,6 +107,8 @@ class _no_spaces_parser(object):
 
     _preferred_formats = ['%Y%m%d%H%M', '%Y%m%d%H%M%S', '%Y%m%d%H%M%S.%f']
 
+    _preferred_formats_ordered_8_digit = ['%m%d%Y', '%d%m%Y', '%Y%m%d', '%Y%d%m', '%m%Y%d', '%d%Y%m']
+
     _timeformats = ['%H%M%S.%f', '%H%M%S', '%H%M', '%H']
 
     period = {
@@ -115,14 +120,14 @@ class _no_spaces_parser(object):
 
     def __init__(self, *args, **kwargs):
 
-        self._all = (self._dateformats +
-                     [x+y for x in self._dateformats for y in self._timeformats] +
-                     self._timeformats)
+        self._all = (
+            self._dateformats + [x + y for x in self._dateformats for y in self._timeformats] + self._timeformats
+        )
 
         self.date_formats = {
             '%m%d%y': (
-                self._preferred_formats +
-                sorted(self._all, key=lambda x: x.lower().startswith('%m%d%y'), reverse=True)
+                self._preferred_formats
+                + sorted(self._all, key=lambda x: x.lower().startswith('%m%d%y'), reverse=True)
             ),
             '%m%y%d': sorted(self._all, key=lambda x: x.lower().startswith('%m%y%d'), reverse=True),
             '%y%m%d': sorted(self._all, key=lambda x: x.lower().startswith('%y%m%d'), reverse=True),
@@ -140,19 +145,34 @@ class _no_spaces_parser(object):
         else:
             return 'year'
 
+    @classmethod
+    def _find_best_matching_date(cls, datestring):
+        for fmt in cls._preferred_formats_ordered_8_digit:
+            try:
+                dt = strptime(datestring, fmt), cls._get_period(fmt)
+                if len(str(dt[0].year)) == 4:
+                    return dt
+            except:
+                pass
+        return None
+
     @classmethod
     def parse(cls, datestring, settings):
         if not no_space_parser_eligibile(datestring):
-            return
+            raise ValueError('Unable to parse date from: %s' % datestring)
 
         datestring = datestring.replace(':', '')
         if not datestring:
-            return
+            raise ValueError("Empty string")
         tokens = tokenizer(datestring)
         if settings.DATE_ORDER:
             order = resolve_date_order(settings.DATE_ORDER)
         else:
             order = cls._default_order
+            if EIGHT_DIGIT.match(datestring):
+                dt = cls._find_best_matching_date(datestring)
+                if dt is not None:
+                    return dt
         nsp = cls()
         ambiguous_date = None
         for token, _ in tokens.tokenize():
@@ -162,6 +182,9 @@ class _no_spaces_parser(object):
                     if len(str(dt[0].year)) < 4:
                         ambiguous_date = dt
                         continue
+
+                    missing = _get_missing_parts(fmt)
+                    _check_strict_parsing(missing, settings)
                     return dt
                 except:
                     pass
@@ -172,7 +195,20 @@ class _no_spaces_parser(object):
                 raise ValueError('Unable to parse date from: %s' % datestring)
 
 
-class _parser(object):
+def _get_missing_error(missing):
+    return 'Fields missing from the date string: {}'.format(', '.join(missing))
+
+
+def _check_strict_parsing(missing, settings):
+    if settings.STRICT_PARSING and missing:
+        raise ValueError(_get_missing_error(missing))
+    elif settings.REQUIRE_PARTS and missing:
+        errors = [part for part in settings.REQUIRE_PARTS if part in missing]
+        if errors:
+            raise ValueError(_get_missing_error(errors))
+
+
+class _parser:
 
     alpha_directives = OrderedDict([
         ('weekday', ['%A', '%a']),
@@ -185,11 +221,10 @@ class _parser(object):
         'year': ['%y', '%Y'],
     }
 
-
     def __init__(self, tokens, settings):
         self.settings = settings
-        self.tokens = list(tokens)
-        self.filtered_tokens = [t for t in self.tokens if t[1] <= 1]
+        self.tokens = [(t[0].strip(), t[1]) for t in list(tokens)]
+        self.filtered_tokens = [(t[0], t[1], i) for i, t in enumerate(self.tokens) if t[1] <= 1]
 
         self.unset_tokens = []
 
@@ -212,52 +247,75 @@ class _parser(object):
 
         skip_index = []
         skip_component = None
-        for index, token_type in enumerate(self.filtered_tokens):
+        skip_tokens = ["t", "year", "hour", "minute"]
+
+        for index, token_type_original_index in enumerate(self.filtered_tokens):
 
             if index in skip_index:
                 continue
 
-            token, type = token_type
+            token, type, original_index = token_type_original_index
 
-            if token in settings.SKIP_TOKENS_PARSER:
+            if token in skip_tokens:
                 continue
 
             if self.time is None:
+                meridian_index = index + 1
+
                 try:
-                    microsecond = MICROSECOND.search(self.filtered_tokens[index+1][0]).group()
-                    _is_after_time_token = token.index(":")
-                    _is_after_period = self.tokens[
-                        self.tokens.index((token, 0)) + 1][0].index('.')
+                    # try case where hours and minutes are separated by a period. Example: 13.20.
+                    _is_before_period = self.tokens[original_index + 1][0] == '.'
+                    _is_after_period = original_index != 0 and self.tokens[original_index - 1][0] == '.'
+
+                    if _is_before_period and not _is_after_period:
+                        index_next_token = index + 1
+                        next_token = self.filtered_tokens[index_next_token][0]
+                        index_in_tokens_for_next_token = self.filtered_tokens[index_next_token][2]
+
+                        next_token_is_last = index_next_token == len(self.filtered_tokens) - 1
+                        if next_token_is_last or self.tokens[index_in_tokens_for_next_token + 1][0] != '.':
+                            new_token = token + ':' + next_token
+                            if re.match(HOUR_MINUTE_REGEX, new_token):
+                                token = new_token
+                                skip_index.append(index + 1)
+                                meridian_index += 1
+                except Exception:
+                    pass
+
+                try:
+                    microsecond = MICROSECOND.search(self.filtered_tokens[index + 1][0]).group()
+                    # Is after time token? raise ValueError if ':' can't be found:
+                    token.index(":")
+                    # Is after period? raise ValueError if '.' can't be found:
+                    self.tokens[self.tokens.index((token, 0)) + 1][0].index('.')
                 except:
                     microsecond = None
 
                 if microsecond:
-                    mindex = index + 2
-                else:
-                    mindex = index + 1
+                    meridian_index += 1
 
                 try:
-                    meridian = MERIDIAN.search(self.filtered_tokens[mindex][0]).group()
+                    meridian = MERIDIAN.search(self.filtered_tokens[meridian_index][0]).group()
                 except:
                     meridian = None
 
                 if any([':' in token, meridian, microsecond]):
                     if meridian and not microsecond:
                         self._token_time = '%s %s' % (token, meridian)
-                        skip_index.append(mindex)
+                        skip_index.append(meridian_index)
                     elif microsecond and not meridian:
                         self._token_time = '%s.%s' % (token, microsecond)
                         skip_index.append(index + 1)
                     elif meridian and microsecond:
                         self._token_time = '%s.%s %s' % (token, microsecond, meridian)
                         skip_index.append(index + 1)
-                        skip_index.append(mindex)
+                        skip_index.append(meridian_index)
                     else:
                         self._token_time = token
                     self.time = lambda: time_parser(self._token_time)
                     continue
 
-            results = self._parse(type, token, settings.FUZZY, skip_component=skip_component)
+            results = self._parse(type, token, skip_component=skip_component)
             for res in results:
                 if len(token) == 4 and res[0] == 'year':
                     skip_component = 'year'
@@ -271,7 +329,6 @@ class _parser(object):
             for token, type, _ in self.unset_tokens:
                 if type == 0:
                     params.update({attr: int(token)})
-                    datetime(**params)
                     setattr(self, '_token_%s' % attr, token)
                     setattr(self, attr, int(token))
 
@@ -295,17 +352,31 @@ class _parser(object):
         try:
             return datetime(**params)
         except ValueError as e:
-            error_text = getattr(e, 'message', None) or e.__str__()
+            error_text = e.__str__()
             error_msgs = ['day is out of range', 'day must be in']
-            if (
-                (error_msgs[0] in error_text or error_msgs[1] in error_text) and
-                not(self._token_day or hasattr(self, '_token_weekday'))
-            ):
-                _, tail = calendar.monthrange(params['year'], params['month'])
-                params['day'] = tail
-                return datetime(**params)
-            else:
-                raise e
+            if error_msgs[0] in error_text or error_msgs[1] in error_text:
+                if not(self._token_day or hasattr(self, '_token_weekday')):
+                    # if day is not available put last day of the month
+                    params['day'] = get_last_day_of_month(params['year'], params['month'])
+                    return datetime(**params)
+                elif not self._token_year and params['day'] == 29 and params['month'] == 2 and \
+                        not calendar.isleap(params['year']):
+                    # fix the year when year is not present and it is 29 of February
+                    params['year'] = self._get_correct_leap_year(self.settings.PREFER_DATES_FROM, params['year'])
+                    return datetime(**params)
+            raise e
+
+    def _get_correct_leap_year(self, prefer_dates_from, current_year):
+        if prefer_dates_from == 'future':
+            return get_next_leap_year(current_year)
+        if prefer_dates_from == 'past':
+            return get_previous_leap_year(current_year)
+
+        # Default case ('current_period'): return closer leap year
+        next_leap_year = get_next_leap_year(current_year)
+        previous_leap_year = get_previous_leap_year(current_year)
+        next_leap_year_is_closer = next_leap_year - current_year < current_year - previous_leap_year
+        return next_leap_year if next_leap_year_is_closer else previous_leap_year
 
     def _set_relative_base(self):
         self.now = self.settings.RELATIVE_BASE
@@ -328,29 +399,14 @@ class _parser(object):
         return strptime(token, directive)
 
     def _results(self):
-        if self.settings.STRICT_PARSING:
-            errors = []
-            if not self.day:
-                errors.append('Day')
-            if not self.month:
-                errors.append('Month')
-            if not self.year:
-                errors.append('Year')
-            if errors:
-                raise ValueError('%s not found in the date string' % ''.join(errors))
-
+        missing = [
+            field for field in ('day', 'month', 'year')
+            if not getattr(self, field)
+        ]
+        _check_strict_parsing(missing, self.settings)
         self._set_relative_base()
 
         time = self.time() if self.time is not None else None
-
-        if self.settings.FUZZY:
-            attr_truth_values = []
-            for attr in ['day', 'month', 'year', 'time']:
-                attr_truth_values.append(getattr(self, attr, False))
-
-            if not any(attr_truth_values):
-                raise ValueError('Nothing date like found')
-
         params = self._get_datetime_obj_params()
 
         if time:
@@ -392,13 +448,35 @@ class _parser(object):
 
             dateobj = dateobj + delta
 
+        # NOTE: If this assert fires, self.now needs to be made offset-aware in a similar
+        # way that dateobj is temporarily made offset-aware.
+        assert not (self.now.tzinfo is None and dateobj.tzinfo is not None),\
+            "`self.now` doesn't have `tzinfo`. Review comment in code for details."
+
+        # Store the original dateobj values so that upon subsequent parsing everything is not
+        # treated as offset-aware if offset awareness is changed.
+        original_dateobj = dateobj
+
+        # Since date comparisons must be either offset-naive or offset-aware, normalize dateobj
+        # to be offset-aware if one or the other is already offset-aware.
+        if self.now.tzinfo is not None and dateobj.tzinfo is None:
+            dateobj = utc.localize(dateobj)
+
         if self.month and not self.year:
-            if self.now < dateobj:
-                if 'past' in self.settings.PREFER_DATES_FROM:
-                    dateobj = dateobj.replace(year=dateobj.year - 1)
-            else:
-                if 'future' in self.settings.PREFER_DATES_FROM:
-                    dateobj = dateobj.replace(year=dateobj.year + 1)
+            try:
+                if self.now < dateobj:
+                    if self.settings.PREFER_DATES_FROM == 'past':
+                        dateobj = dateobj.replace(year=dateobj.year - 1)
+                else:
+                    if self.settings.PREFER_DATES_FROM == 'future':
+                        dateobj = dateobj.replace(year=dateobj.year + 1)
+            except ValueError as e:
+                if dateobj.day == 29 and dateobj.month == 2:
+                    valid_year = self._get_correct_leap_year(
+                        self.settings.PREFER_DATES_FROM, dateobj.year)
+                    dateobj = dateobj.replace(year=valid_year)
+                else:
+                    raise e
 
         if self._token_year and len(self._token_year[0]) == 2:
             if self.now < dateobj:
@@ -419,27 +497,24 @@ class _parser(object):
                 if self.now.time() > dateobj.time():
                     dateobj = dateobj + timedelta(days=1)
 
+        # Reset dateobj to the original value, thus removing any offset awareness that may
+        # have been set earlier.
+        dateobj = dateobj.replace(tzinfo=original_dateobj.tzinfo)
+
         return dateobj
 
     def _correct_for_day(self, dateobj):
         if (
-            getattr(self, '_token_day', None) or
-            getattr(self, '_token_weekday', None) or
-            getattr(self, '_token_time', None)
+            getattr(self, '_token_day', None)
+            or getattr(self, '_token_weekday', None)
+            or getattr(self, '_token_time', None)
         ):
             return dateobj
 
-        _, tail = calendar.monthrange(dateobj.year, dateobj.month)
-        options = {
-            'first': 1,
-            'last': tail,
-            'current': self.now.day
-        }
-
-        try:
-            return dateobj.replace(day=options[self.settings.PREFER_DAY_OF_MONTH])
-        except ValueError:
-            return dateobj.replace(day=options['last'])
+        dateobj = set_correct_day_from_settings(
+            dateobj, self.settings, current_day=self.now.day
+        )
+        return dateobj
 
     @classmethod
     def parse(cls, datestring, settings):
@@ -456,7 +531,7 @@ class _parser(object):
 
         return dateobj, period
 
-    def _parse(self, type, token, fuzzy, skip_component=None):
+    def _parse(self, type, token, skip_component=None):
 
         def set_and_return(token, type, component, dateobj, skip_date_order=False):
             if not skip_date_order:
@@ -487,10 +562,7 @@ class _parser(object):
                     except ValueError:
                         pass
             else:
-                if not fuzzy:
-                    raise ValueError('Unable to parse: %s' % token)
-                else:
-                    return []
+                raise ValueError('Unable to parse: %s' % token)
 
         def parse_alpha(token, skip_component=None):
             type = 1
@@ -513,25 +585,21 @@ class _parser(object):
                     except:
                         pass
             else:
-                if not fuzzy:
-                    raise ValueError('Unable to parse: %s' % token)
-                else:
-                    return []
+                raise ValueError('Unable to parse: %s' % token)
 
         handlers = {0: parse_number, 1: parse_alpha}
         return handlers[type](token, skip_component)
 
 
-class tokenizer(object):
-    digits = u'0123456789:'
-    letters = u'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
-    nonwords = u"./\()\"',.;<>~!@#$%^&*|+=[]{}`~?-     "
+class tokenizer:
+    digits = '0123456789:'
+    letters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
 
-    def _isletter(self, tkn): return tkn in self.letters
+    def _isletter(self, tkn):
+        return tkn in self.letters
 
-    def _isdigit(self, tkn): return tkn in self.digits
-
-    def _isnonword(self, tkn): return tkn in self.nonwords
+    def _isdigit(self, tkn):
+        return tkn in self.digits
 
     def __init__(self, ds):
         self.instream = StringIO(ds)
@@ -543,10 +611,7 @@ class tokenizer(object):
         if self._isletter(chara):
             return 1, not self._isletter(charb)
 
-        if self._isnonword(chara):
-            return 2, not self._isnonword(charb)
-
-        return '', True
+        return 2, self._isdigit(charb) or self._isletter(charb)
 
     def tokenize(self):
         token = ''
