@@ -432,8 +432,8 @@ class TestTranslateSearch(BaseTestCase):
                     'інтервенції в Маньчжурію 13 вересня 1931, початок другої японсько-китайської війни 7 '
                     'липня 1937 року та початок угорсько-української війни 14 березня 1939 року.',
               [('13 вересня 1931', datetime.datetime(1931, 9, 13, 0, 0)),
-               ('7 липня 1937', datetime.datetime(1937, 7, 7, 0, 0)),
-               ('14 березня 1939', datetime.datetime(1939, 3, 14, 0, 0))],
+               ('7 липня 1937 року', datetime.datetime(1937, 7, 7, 0, 0)),
+               ('14 березня 1939 року', datetime.datetime(1939, 3, 14, 0, 0))],
               settings={'RELATIVE_BASE': datetime.datetime(2000, 1, 1)}),
 
         # Vietnamese
@@ -461,15 +461,15 @@ class TestTranslateSearch(BaseTestCase):
                ('October', datetime.datetime(2014, 10, datetime.datetime.utcnow().day, 0, 0)),
                ('Friday, 21', datetime.datetime(2014, 10, 21, 0, 0))]),
         param('en', """May 2020
-                    June 2020
+                    July 2020
                     2023
                     January UTC
                     June 5 am utc
                     June 23th 5 pm EST
                     May 31, 8am UTC""",
               [('May 2020', datetime.datetime(2020, 5, datetime.datetime.utcnow().day, 0, 0)),
-               ('June 2020', datetime.datetime(2020, 6, datetime.datetime.utcnow().day, 0, 0)),
-               ('2023', datetime.datetime(2023, 6, datetime.datetime.utcnow().day, 0, 0)),
+               ('July 2020', datetime.datetime(2020, 7, datetime.datetime.utcnow().day, 0, 0)),
+               ('2023', datetime.datetime(2023, 7, datetime.datetime.utcnow().day, 0, 0)),
                ('January UTC', datetime.datetime(2023, 1, datetime.datetime.utcnow().day, 0, 0, tzinfo=pytz.utc)),
                ('June 5 am utc', datetime.datetime(2023, 6, 5, 0, 0, tzinfo=pytz.utc)),
                ('June 23th 5 pm EST', datetime.datetime(2023, 6, 23, 17, 0, tzinfo=pytz.timezone("EST"))),
@@ -481,6 +481,8 @@ class TestTranslateSearch(BaseTestCase):
               [('19 марта 2001', datetime.datetime(2001, 3, 19, 0, 0)),
                ('20 марта', datetime.datetime(2001, 3, 20, 0, 0)),
                ('21 марта', datetime.datetime(2001, 3, 21, 0, 0))]),
+        param('ru', 'Андрей Дмитриевич Сахаров скончался 14 декабря в 1989 году от внезапной остановки сердца.',
+              [('14 декабря в 1989 году', datetime.datetime(1989, 12, 14, 0, 0))]),
         # relative dates
         param('ru', '19 марта 2001. Сегодня был хороший день. 2 дня назад был хороший день. '
                     'Вчера тоже был хороший день.',
